Package: IceSat2R
Type: Package
Title: 'ICESat-2' Altimeter Data using R
Version: 1.1.0
Date: 2026-06-12
Authors@R: c( person(given = "Lampros", family = "Mouselimis", email = "mouselimislampros@gmail.com", role = c("aut", "cre"), comment = c(ORCID = "https://orcid.org/0000-0002-8024-1546")), person("OpenAltimetry.org", role = "cph"))
Description: Programmatic connection to the 'OpenAltimetry' API <https://openaltimetry.earthdatacloud.nasa.gov/data/openapi/swagger-ui/index.html/> to download and process 'ATL03' (Global 'Geolocated' Photon Data), 'ATL06' (Land Ice Height), 'ATL07' (Sea Ice Height), 'ATL08' (Land and Vegetation Height), 'ATL10' (Sea Ice 'Freeboard'), 'ATL12' (Ocean Surface Height) and 'ATL13' (Inland Water Surface Height) 'ICESat-2' Altimeter Data. The user has the option to download the data by selecting a bounding box from a 1- or 5-degree grid globally utilizing a shiny application. The 'ICESat-2' mission collects 'altimetry' data of the Earth's surface. The sole instrument on 'ICESat-2' is the Advanced Topographic Laser Altimeter System (ATLAS) instrument that measures ice sheet elevation change and sea ice thickness, while also generating an estimate of global vegetation biomass. 'ICESat-2' continues the important observations of ice-sheet elevation change, sea-ice 'freeboard', and vegetation canopy height begun by 'ICESat' in 2003.
License: GPL-3
Encoding: UTF-8
URL: https://github.com/mlampros/IceSat2R,
        https://mlampros.github.io/IceSat2R/
BugReports: https://github.com/mlampros/IceSat2R/issues
Depends: R(>= 4.1.0)
Imports: glue, sf, lwgeom, units, data.table, httr, utils, foreach,
        tools, doParallel, magrittr, leaflet, leafgl, htmltools,
        htmlwidgets, leafsync, miniUI, shiny, rnaturalearth, lubridate,
        rvest, withr
Suggests: rmarkdown, knitr, DT, mapview, grDevices, stargazer,
        reshape2, plotly, geodist, CopernicusDEM, terra, testthat (>=
        3.0.0)
RoxygenNote: 7.3.2
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-06-12 12:56:55 UTC; lampros
Author: Lampros Mouselimis [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-8024-1546>),
  OpenAltimetry.org [cph]
Maintainer: Lampros Mouselimis <mouselimislampros@gmail.com>
Repository: CRAN
Date/Publication: 2026-06-12 14:00:02 UTC
Built: R 4.5.2; ; 2026-06-12 16:40:27 UTC; unix
