# Generated by roxygen2: do not edit by hand

export(available_RGTs)
export(available_nominal_orbits)
export(degrees_to_global_grid)
export(download_file)
export(getTracks)
export(get_atlas_data)
export(get_level3a_data)
export(latest_orbits)
export(overall_mission_orbits)
export(revisit_time_RGTs)
export(select_aoi_global_grid)
export(time_specific_orbits)
export(verify_RGTs)
export(vsi_kml_from_zip)
export(vsi_nominal_orbits_wkt)
export(vsi_time_specific_orbits_wkt)
import(foreach)
import(lwgeom)
import(magrittr)
importFrom(data.table,data.table)
importFrom(data.table,fread)
importFrom(data.table,rbindlist)
importFrom(data.table,setDT)
importFrom(doParallel,registerDoParallel)
importFrom(glue,glue)
importFrom(htmlwidgets,JS)
importFrom(httr,GET)
importFrom(httr,content)
importFrom(httr,http_error)
importFrom(leafgl,addGlPolygons)
importFrom(leafgl,addGlPolylines)
importFrom(leaflet,addLayersControl)
importFrom(leaflet,addProviderTiles)
importFrom(leaflet,filterNULL)
importFrom(leaflet,getMapData)
importFrom(leaflet,invokeMethod)
importFrom(leaflet,layersControlOptions)
importFrom(leaflet,leaflet)
importFrom(leaflet,leafletDependencies)
importFrom(leaflet,providers)
importFrom(leafsync,sync)
importFrom(lubridate,yday)
importFrom(lubridate,ymd)
importFrom(miniUI,gadgetTitleBar)
importFrom(miniUI,miniContentPanel)
importFrom(miniUI,miniPage)
importFrom(miniUI,miniTitleBarButton)
importFrom(rnaturalearth,ne_countries)
importFrom(rvest,html_elements)
importFrom(rvest,read_html)
importFrom(sf,gdal_utils)
importFrom(sf,sf_use_s2)
importFrom(sf,st_area)
importFrom(sf,st_as_sf)
importFrom(sf,st_as_sfc)
importFrom(sf,st_bbox)
importFrom(sf,st_cast)
importFrom(sf,st_crs)
importFrom(sf,st_geometry)
importFrom(sf,st_intersects)
importFrom(sf,st_layers)
importFrom(sf,st_make_grid)
importFrom(sf,st_read)
importFrom(sf,st_zm)
importFrom(shiny,dialogViewer)
importFrom(shiny,isTruthy)
importFrom(shiny,observeEvent)
importFrom(shiny,runGadget)
importFrom(shiny,stopApp)
importFrom(tools,file_ext)
importFrom(units,set_units)
importFrom(utils,download.file)
importFrom(utils,flush.console)
importFrom(utils,globalVariables)
importFrom(utils,unzip)
importFrom(withr,with_options)
